import os
import toml
import logging.config
import collections.abc

from pathlib import Path
from werkzeug.middleware.proxy_fix import ProxyFix
from flaschengeist import _module_path, logger


# Default config:
config = {}


def update_dict(d, u):
    for k, v in u.items():
        if isinstance(v, collections.abc.Mapping):
            d[k] = update_dict(d.get(k, {}), v)
        else:
            d[k] = v
    return d


def read_configuration():
    global config

    paths = [_module_path, Path.home() / ".config"]
    if "FLASCHENGEIST_CONF" in os.environ:
        paths.append(Path(os.environ.get("FLASCHENGEIST_CONF")))

    for loc in paths:
        try:
            with (loc / "flaschengeist.toml").open() as source:
                print("Reading config file from >{}<".format(loc))
                update_dict(config, toml.load(source))
        except IOError:
            pass


def configure_app(app, test_config=None):
    global config
    if test_config is None:
        read_configuration()
    else:
        update_dict(config, test_config)
    # Always enable this builtin plugins!
    update_dict(config, {"auth": {"enabled": True}, "roles": {"enabled": True}, "users": {"enabled": True}})

    logger_config = toml.load(_module_path / "logging.toml")
    if "LOGGING" in config:
        if "level" in config["LOGGING"]:
            logger_config["loggers"]["flaschengeist"] = {"level": config["LOGGING"]["level"]}
        if "file" in config["LOGGING"]:
            logger_config["root"]["handlers"].append("file")
            logger_config["handlers"]["file"]["filename"] = config["LOGGING"]["file"]
            path = Path(config["LOGGING"]["file"])
            path.parent.mkdir(parents=True, exist_ok=True)
    logging.config.dictConfig(logger_config)

    if "secret_key" not in config["FLASCHENGEIST"]:
        logger.warning("No secret key was configured, please configure one for production systems!")
        app.config["SECRET_KEY"] = "0a657b97ef546da90b2db91862ad4e29"
    else:
        app.config["SECRET_KEY"] = config["FLASCHENGEIST"]["secret_key"]

    if test_config is None:
        app.config["SQLALCHEMY_DATABASE_URI"] = "mysql://{user}:{passwd}@{host}/{database}".format(
            user=config["DATABASE"]["user"],
            passwd=config["DATABASE"]["password"],
            host=config["DATABASE"]["host"],
            database=config["DATABASE"]["database"],
        )
    else:
        app.config["SQLALCHEMY_DATABASE_URI"] = f"sqlite+pysqlite://{config['DATABASE']['file_path']}"
    app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False

    if "root" in config["FLASCHENGEIST"]:
        logger.debug("Setting application root to >{}<".format(config["FLASCHENGEIST"]["root"]))
        app.config["APPLICATION_ROOT"] = config["FLASCHENGEIST"]["root"]
    if config["FLASCHENGEIST"].get("proxy", False):
        logger.debug("Fixing wsgi_app for using behind a proxy server")
        app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)
